@@ -19,11 +19,33 @@ Revision History
 
 --*/
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include "efidebug.h"
 #include "efipart.h"
-#include "efilibplat.h"
+#if defined(_M_X64) || defined(__x86_64__) || defined(__amd64__)
+#include "x86_64/efilibplat.h"
+#elif defined(_M_IX86) || defined(__i386__)
+#include "ia32/efilibplat.h"
+#elif defined(_M_IA64) || defined(__ia64__)
+#include "ia64/efilibplat.h"
+#elif defined (_M_ARM64) || defined(__aarch64__)
+#include "aarch64/efilibplat.h"
+#elif defined (_M_ARM) || defined(__arm__)
+#include "arm/efilibplat.h"
+#elif defined (_M_MIPS64) || defined(__mips64__) || defined(__mips64)
+#include "mips64el/efilibplat.h"
+#elif defined (__riscv) && __riscv_xlen == 64
+#include "riscv64/efilibplat.h"
+#elif defined (__loongarch64)
+#include "loongarch64/efilibplat.h"
+#endif
+#include "legacy/efilib.h"
 #include "efilink.h"
 #include "efirtlib.h"
+#include "efistdarg.h"
 #include "pci22.h"
 #include "libsmbios.h"
 
@@ -32,39 +54,106 @@ Revision History
 //
 
 extern EFI_SYSTEM_TABLE         *ST;
+#define gST                      ST
 extern EFI_BOOT_SERVICES        *BS;
+#define gBS                      BS
 extern EFI_RUNTIME_SERVICES     *RT;
+#define gRT                      RT
 
-extern EFI_GUID DevicePathProtocol;
-extern EFI_GUID LoadedImageProtocol;
-extern EFI_GUID TextInProtocol;
-extern EFI_GUID TextOutProtocol;
-extern EFI_GUID BlockIoProtocol;
-extern EFI_GUID DiskIoProtocol;
-extern EFI_GUID FileSystemProtocol;
-extern EFI_GUID LoadFileProtocol;
-extern EFI_GUID DeviceIoProtocol;
+extern EFI_GUID gEfiDevicePathProtocolGuid;
+#define DevicePathProtocol gEfiDevicePathProtocolGuid
+extern EFI_GUID gEfiDevicePathToTextProtocolGuid;
+#define DevicePathToTextProtocol gEfiDevicePathToTextProtocolGuid
+extern EFI_GUID gEfiDevicePathFromTextProtocolGuid;
+#define DevicePathFromTextProtocol gEfiDevicePathFromTextProtocolGuid
+extern EFI_GUID gEfiDevicePathUtilitiesProtocolGuid;
+#define DevicePathUtilitiesProtocol gEfiDevicePathUtilitiesProtocolGuid
+extern EFI_GUID gEfiLoadedImageProtocolGuid;
+#define LoadedImageProtocol gEfiLoadedImageProtocolGuid
+extern EFI_GUID gEfiSimpleTextInProtocolGuid;
+#define TextInProtocol gEfiSimpleTextInProtocolGuid
+extern EFI_GUID gEfiSimpleTextOutProtocolGuid;
+#define TextOutProtocol gEfiSimpleTextOutProtocolGuid
+extern EFI_GUID gEfiGraphicsOutputProtocolGuid;
+#define GraphicsOutputProtocol gEfiGraphicsOutputProtocolGuid
+extern EFI_GUID gEfiEdidDiscoveredProtocolGuid;
+#define EdidDiscoveredProtocol gEfiEdidDiscoveredProtocolGuid
+extern EFI_GUID gEfiEdidActiveProtocolGuid;
+#define EdidActiveProtocol gEfiEdidActiveProtocolGuid
+extern EFI_GUID gEfiEdidOverrideProtocolGuid;
+#define EdidOverrideProtocol gEfiEdidOverrideProtocolGuid
+extern EFI_GUID gEfiBlockIoProtocolGuid;
+#define BlockIoProtocol gEfiBlockIoProtocolGuid
+extern EFI_GUID gEfiBlockIo2ProtocolGuid;
+#define BlockIo2Protocol gEfiBlockIo2ProtocolGuid
+extern EFI_GUID gEfiDiskIoProtocolGuid;
+#define DiskIoProtocol gEfiDiskIoProtocolGuid
+extern EFI_GUID gEfiDiskIo2ProtocolGuid;
+#define DiskIo2Protocol gEfiDiskIo2ProtocolGuid
+extern EFI_GUID gEfiSimpleFileSystemProtocolGuid;
+#define FileSystemProtocol gEfiSimpleFileSystemProtocolGuid
+extern EFI_GUID gEfiLoadFileProtocolGuid;
+#define LoadFileProtocol gEfiLoadFileProtocolGuid
+extern EFI_GUID gEfiDeviceIoProtocolGuid;
+#define DeviceIoProtocol gEfiDeviceIoProtocolGuid
 extern EFI_GUID VariableStoreProtocol;
 extern EFI_GUID LegacyBootProtocol;
-extern EFI_GUID UnicodeCollationProtocol;
-extern EFI_GUID SerialIoProtocol;
+extern EFI_GUID gEfiUnicodeCollationProtocolGuid;
+#define UnicodeCollationProtocol gEfiUnicodeCollationProtocolGuid
+extern EFI_GUID gEfiSerialIoProtocolGuid;
+#define SerialIoProtocol gEfiSerialIoProtocolGuid
 extern EFI_GUID VgaClassProtocol;
 extern EFI_GUID TextOutSpliterProtocol;
 extern EFI_GUID ErrorOutSpliterProtocol;
 extern EFI_GUID TextInSpliterProtocol;
-extern EFI_GUID SimpleNetworkProtocol;
-extern EFI_GUID PxeBaseCodeProtocol;
-extern EFI_GUID PxeCallbackProtocol;
-extern EFI_GUID NetworkInterfaceIdentifierProtocol;
-extern EFI_GUID UiProtocol;
+extern EFI_GUID gEfiSimpleNetworkProtocolGuid;
+#define SimpleNetworkProtocol gEfiSimpleNetworkProtocolGuid
+extern EFI_GUID gEfiPxeBaseCodeProtocolGuid;
+#define PxeBaseCodeProtocol gEfiPxeBaseCodeProtocolGuid
+extern EFI_GUID gEfiPxeBaseCodeCallbackProtocolGuid;
+#define PxeCallbackProtocol gEfiPxeBaseCodeCallbackProtocolGuid
+extern EFI_GUID gEfiNetworkInterfaceIdentifierProtocolGuid;
+#define NetworkInterfaceIdentifierProtocol gEfiNetworkInterfaceIdentifierProtocolGuid
+extern EFI_GUID gEFiUiInterfaceProtocolGuid;
+#define UiProtocol gEFiUiInterfaceProtocolGuid
 extern EFI_GUID InternalShellProtocol;
+extern EFI_GUID gEfiPciIoProtocolGuid;
+#define PciIoProtocol gEfiPciIoProtocolGuid
+extern EFI_GUID gEfiPciRootBridgeIoProtocolGuid;
+extern EFI_GUID gEfiDriverBindingProtocolGuid;
+#define DriverBindingProtocol gEfiDriverBindingProtocolGuid
+extern EFI_GUID gEfiComponentNameProtocolGuid;
+#define ComponentNameProtocol gEfiComponentNameProtocolGuid
+extern EFI_GUID gEfiComponentName2ProtocolGuid;
+#define ComponentName2Protocol gEfiComponentName2ProtocolGuid
+extern EFI_GUID gEfiHashProtocolGuid;
+#define HashProtocol gEfiHashProtocolGuid
+extern EFI_GUID gEfiPlatformDriverOverrideProtocolGuid;
+#define PlatformDriverOverrideProtocol gEfiPlatformDriverOverrideProtocolGuid
+extern EFI_GUID gEfiBusSpecificDriverOverrideProtocolGuid;
+#define BusSpecificDriverOverrideProtocol gEfiBusSpecificDriverOverrideProtocolGuid
+extern EFI_GUID gEfiDriverFamilyOverrideProtocolGuid;
+#define DriverFamilyOverrideProtocol gEfiDriverFamilyOverrideProtocolGuid
+extern EFI_GUID gEfiEbcProtocolGuid;
+
+extern EFI_GUID gEfiMpServicesProtocolGuid;
+#define MpServicesProtocol gEfiMpServicesProtocolGuid
+
+extern EFI_GUID gEfiGlobalVariableGuid;
+#define EfiGlobalVariable gEfiGlobalVariableGuid
+extern EFI_GUID gEfiFileInfoGuid;
+#define GenericFileInfo gEfiFileInfoGuid
+extern EFI_GUID gEfiFileSystemInfoGuid;
+#define FileSystemInfo gEfiFileSystemInfoGuid
+extern EFI_GUID gEfiFileSystemVolumeLabelInfoIdGuid;
+#define FileSystemVolumeLabelInfo gEfiFileSystemVolumeLabelInfoIdGuid
+extern EFI_GUID gEfiPcAnsiGuid;
+#define PcAnsiProtocol gEfiPcAnsiGuid
+extern EFI_GUID gEfiVT100Guid;
+#define Vt100Protocol gEfiVT100Guid
+extern EFI_GUID gEfiVT100PlusGuid;
+extern EFI_GUID gEfiVTUTF8Guid;
 
-extern EFI_GUID EfiGlobalVariable;
-extern EFI_GUID GenericFileInfo;
-extern EFI_GUID FileSystemInfo;
-extern EFI_GUID FileSystemVolumeLabelInfo;
-extern EFI_GUID PcAnsiProtocol;
-extern EFI_GUID Vt100Protocol;
 extern EFI_GUID NullGuid;
 extern EFI_GUID UnknownDevice;
 
@@ -73,29 +162,45 @@ extern EFI_GUID EfiPartTypeLegacyMbrGuid;
 
 extern EFI_GUID MpsTableGuid;
 extern EFI_GUID AcpiTableGuid;
-extern EFI_GUID SMBIOSTableGuid;
+extern EFI_GUID gEfiSmbiosTableGuid;
+#define SMBIOSTableGuid gEfiSmbiosTableGuid
+extern EFI_GUID gEfiSmbios3TableGuid;
+#define SMBIOS3TableGuid gEfiSmbios3TableGuid
 extern EFI_GUID SalSystemTableGuid;
+extern EFI_GUID EfiDtbTableGuid;
+
+extern EFI_GUID SimplePointerProtocol;
+extern EFI_GUID AbsolutePointerProtocol;
+
+extern EFI_GUID gEfiDebugImageInfoTableGuid;
+extern EFI_GUID gEfiDebugSupportProtocolGuid;
+
+extern EFI_GUID SimpleTextInputExProtocol;
+
+extern EFI_GUID ShellProtocolGuid;
+extern EFI_GUID ShellParametersProtocolGuid;
+extern EFI_GUID ShellDynamicCommandProtocolGuid;
 
 //
 // EFI Variable strings
 //
 #define LOAD_OPTION_ACTIVE      0x00000001
 
-#define VarLanguageCodes       L"LangCodes"
-#define VarLanguage            L"Lang"
-#define VarTimeout             L"Timeout"
-#define VarConsoleInp          L"ConIn"
-#define VarConsoleOut          L"ConOut"
-#define VarErrorOut            L"ErrOut"
-#define VarBootOption          L"Boot%04x"
-#define VarBootOrder           L"BootOrder"
-#define VarBootNext            L"BootNext"
-#define VarBootCurrent         L"BootCurrent"
-#define VarDriverOption        L"Driver%04x"
-#define VarDriverOrder         L"DriverOrder"
-#define VarConsoleInpDev       L"ConInDev"
-#define VarConsoleOutDev       L"ConOutDev"
-#define VarErrorOutDev         L"ErrOutDev"
+#define VarLanguageCodes       u"LangCodes"
+#define VarLanguage            u"Lang"
+#define VarTimeout             u"Timeout"
+#define VarConsoleInp          u"ConIn"
+#define VarConsoleOut          u"ConOut"
+#define VarErrorOut            u"ErrOut"
+#define VarBootOption          u"Boot%04x"
+#define VarBootOrder           u"BootOrder"
+#define VarBootNext            u"BootNext"
+#define VarBootCurrent         u"BootCurrent"
+#define VarDriverOption        u"Driver%04x"
+#define VarDriverOrder         u"DriverOrder"
+#define VarConsoleInpDev       u"ConInDev"
+#define VarConsoleOutDev       u"ConOutDev"
+#define VarErrorOutDev         u"ErrOutDev"
 
 #define LanguageCodeEnglish    "eng"
 
@@ -110,12 +215,10 @@ extern EFI_DEVICE_PATH EndInstanceDevicePath[];
 extern EFI_MEMORY_TYPE PoolAllocationType;
 
 //
-// STATIC - Name is internal to the module
 // INTERNAL - Name is internal to the component (i.e., directory)
 // BOOTSERVCE - Name of a boot service function
 //
 
-#define STATIC
 #define INTERNAL
 #define BOOTSERVICE
 
@@ -139,6 +242,19 @@ EFIDebugVariable (
     VOID
     );
 
+VOID
+Exit(
+    IN EFI_STATUS   ExitStatus,
+    IN UINTN        ExitDataSize,
+    IN CHAR16       *ExitData OPTIONAL
+    );
+
+INTN
+GetShellArgcArgv(
+    EFI_HANDLE ImageHandle,
+    CHAR16 **Argv[]  /* Statically allocated */
+    );
+
 VOID
 SetCrc (
     IN OUT EFI_TABLE_HEADER *Hdr
@@ -175,44 +291,51 @@ ZeroMem (
     IN UINTN     Size
     );
 
-VOID
+VOID EFIAPI
 SetMem (
     IN VOID     *Buffer,
     IN UINTN    Size,
-    IN UINT8    Value    
+    IN UINT8    Value
     );
 
-VOID
-CopyMem (
+VOID EFIAPI
+CopyMem_1 (
     IN VOID     *Dest,
     IN VOID     *Src,
     IN UINTN    len
     );
 
+VOID EFIAPI
+CopyMemC (
+    IN VOID     *Dest,
+    IN CONST VOID     *Src,
+    IN UINTN    len
+    );
+
 INTN
 CompareMem (
-    IN VOID     *Dest,
-    IN VOID     *Src,
+    IN CONST VOID     *Dest,
+    IN CONST VOID     *Src,
     IN UINTN    len
     );
 
 INTN
 StrCmp (
-    IN CHAR16   *s1,
-    IN CHAR16   *s2
+    IN CONST CHAR16   *s1,
+    IN CONST CHAR16   *s2
     );
 
 INTN
 StrnCmp (
-    IN CHAR16   *s1,
-    IN CHAR16   *s2,
+    IN CONST CHAR16   *s1,
+    IN CONST CHAR16   *s2,
     IN UINTN    len
     );
 
 INTN
 StriCmp (
-    IN CHAR16   *s1,
-    IN CHAR16   *s2
+    IN CONST CHAR16   *s1,
+    IN CONST CHAR16   *s2
     );
 
 VOID
@@ -228,65 +351,105 @@ StrUpr (
 VOID
 StrCpy (
     IN CHAR16   *Dest,
-    IN CHAR16    *Src
+    IN CONST CHAR16    *Src
+    );
+
+VOID
+StrnCpy (
+    IN CHAR16   *Dest,
+    IN CONST CHAR16    *Src,
+    IN UINTN     Len
+    );
+
+CHAR16 *
+StpCpy (
+    IN CHAR16   *Dest,
+    IN CONST CHAR16    *Src
+    );
+
+CHAR16 *
+StpnCpy (
+    IN CHAR16   *Dest,
+    IN CONST CHAR16    *Src,
+    IN UINTN     Len
     );
 
 VOID
 StrCat (
     IN CHAR16   *Dest,
-    IN CHAR16   *Src
+    IN CONST CHAR16   *Src
+    );
+
+VOID
+StrnCat (
+    IN CHAR16   *Dest,
+    IN CONST CHAR16   *Src,
+    IN UINTN     Len
     );
 
 UINTN
 StrLen (
-    IN CHAR16   *s1
+    IN CONST CHAR16   *s1
+    );
+
+UINTN
+StrnLen (
+    IN CONST CHAR16   *s1,
+    IN UINTN           Len
     );
 
 UINTN
 StrSize (
-    IN CHAR16   *s1
+    IN CONST CHAR16   *s1
     );
 
 CHAR16 *
 StrDuplicate (
-    IN CHAR16   *Src
+    IN CONST CHAR16   *Src
     );
 
 UINTN
-strlena (
-    IN CHAR8    *s1
-    );
-    
-UINTN
-strcmpa (
-    IN CHAR8    *s1,
-    IN CHAR8    *s2
+AsciiStrLen (
+    IN CONST CHAR8    *s1
     );
 
 UINTN
-strncmpa (
-    IN CHAR8    *s1,
-    IN CHAR8    *s2,
-    IN UINTN    len
+AsciiStrCmp (
+    IN CONST CHAR8    *s1,
+    IN CONST CHAR8    *s2
     );
 
+UINTN
+AsciiStrnCmp (
+    IN CONST CHAR8    *s1,
+    IN CONST CHAR8    *s2,
+    IN UINTN          len
+    );
+
+//
+// For compatibility with previous gnu-efi versions
+//
+#define strlena AsciiStrLen
+#define strcmpa AsciiStrCmp
+#define strncmpa AsciiStrnCmp
+
 UINTN
 xtoi (
-    CHAR16      *str
+    CONST CHAR16      *str
     );
 
 UINTN
 Atoi (
-    CHAR16  *str
+    CONST CHAR16  *str
     );
 
-BOOLEAN 
+BOOLEAN
 MetaMatch (
     IN CHAR16   *String,
     IN CHAR16   *Pattern
     );
 
-BOOLEAN 
+BOOLEAN
 MetaiMatch (
     IN CHAR16   *String,
     IN CHAR16   *Pattern
@@ -334,10 +497,11 @@ ReleaseLock (
     );
 
 
-INTN
-CompareGuid(
-    IN EFI_GUID     *Guid1,
-    IN EFI_GUID     *Guid2
+BOOLEAN
+EFIAPI
+CompareGuid_1 (
+    IN CONST EFI_GUID     *Guid1,
+    IN CONST EFI_GUID     *Guid2
     );
 
 VOID *
@@ -351,11 +515,12 @@ AllocateZeroPool (
     );
 
 VOID *
-ReallocatePool (
-    IN VOID                 *OldPool,
-    IN UINTN                OldSize,
-    IN UINTN                NewSize
-    );
+EFIAPI
+ReallocatePool_1 (
+        IN UINTN  OldSize,
+        IN UINTN  NewSize,
+        IN VOID   *OldPool  OPTIONAL
+);
 
 VOID
 FreePool (
@@ -386,21 +551,106 @@ IInput (
 
 UINTN
 Print (
-    IN CHAR16   *fmt,
+    IN CONST CHAR16   *fmt,
     ...
     );
 
 UINTN
-SPrint (
-    OUT CHAR16  *Str,
-    IN UINTN    StrSize,
-    IN CHAR16   *fmt,
+VPrint (
+    IN CONST CHAR16   *fmt,
+    va_list           args
+    );
+
+#if !defined(_MSC_VER)
+UINTN
+EFIAPI
+MS_VPrint (
+    IN CONST CHAR16   *fmt,
+    ms_va_list           args
+    );
+#else
+EFI_INTERNAL
+UINTN
+EFIAPI
+MS_VPrint (
+    IN CONST CHAR16   *fmt,
+    ms_va_list           args
+) {
+   return VPrint(fmt, args);
+}
+#endif
+
+
+UINTN
+UnicodeSPrint (
+    OUT CHAR16        *Str,
+    IN UINTN          StrSize,
+    IN CONST CHAR16   *fmt,
     ...
     );
+    
+UINTN
+UnicodeVSPrint (
+    OUT CHAR16        *Str,
+    IN UINTN          StrSize,
+    IN CONST CHAR16   *fmt,
+    va_list           args
+    );
+
+
+#if !defined(_MSC_VER)
+UINTN
+EFIAPI
+MS_UnicodeVSPrint (
+    OUT CHAR16        *Str,
+    IN UINTN          StrSize,
+    IN CONST CHAR16   *fmt,
+    ms_va_list           args
+    );
+#else
+EFI_INTERNAL
+UINTN
+EFIAPI
+MS_UnicodeVSPrint (
+    OUT CHAR16        *Str,
+    IN UINTN          StrSize,
+    IN CONST CHAR16   *fmt,
+    ms_va_list           args
+) {
+    return UnicodeVSPrint(Str, StrSize, fmt, args);
+}
+#endif
+
+
+CHAR16 *
+VPoolPrint (
+    IN CONST CHAR16     *fmt,
+    va_list             args
+    );
+
+#if !defined(_MSC_VER)
+CHAR16 *
+EFIAPI
+MS_VPoolPrint (
+    IN CONST CHAR16     *fmt,
+    ms_va_list             args
+    );
+#else
+EFI_INTERNAL
+CHAR16 *
+EFIAPI
+MS_VPoolPrint (
+    IN CONST CHAR16     *fmt,
+    ms_va_list             args
+) {
+    return VPoolPrint(fmt, args);
+}
+#endif
+
 
 CHAR16 *
 PoolPrint (
-    IN CHAR16           *fmt,
+    IN CONST CHAR16     *fmt,
     ...
     );
 
@@ -413,22 +663,22 @@ typedef struct {
 CHAR16 *
 CatPrint (
     IN OUT POOL_PRINT   *Str,
-    IN CHAR16           *fmt,
+    IN CONST CHAR16     *fmt,
     ...
     );
 
 UINTN
 PrintAt (
-    IN UINTN    Column,
-    IN UINTN    Row,
-    IN CHAR16   *fmt,
+    IN UINTN         Column,
+    IN UINTN         Row,
+    IN CONST CHAR16  *fmt,
     ...
     );
 
 UINTN
 IPrint (
     IN SIMPLE_TEXT_OUTPUT_INTERFACE    *Out,
-    IN CHAR16                          *fmt,
+    IN CONST CHAR16                    *fmt,
     ...
     );
 
@@ -437,16 +687,40 @@ IPrintAt (
     IN SIMPLE_TEXT_OUTPUT_INTERFACE     *Out,
     IN UINTN                            Column,
     IN UINTN                            Row,
-    IN CHAR16                           *fmt,
+    IN CONST CHAR16                     *fmt,
     ...
     );
 
 UINTN
-APrint (
-    IN CHAR8    *fmt,
+AsciiPrint (
+    IN CONST CHAR8    *fmt,
     ...
     );
 
+UINTN
+AsciiVSPrint(
+    OUT CHAR8         *Str,
+    IN  UINTN         StrSize,
+    IN  CONST CHAR8   *fmt,
+    va_list           args
+);
+
+UINTN
+EFIAPI
+MS_AsciiVSPrint(
+    OUT CHAR8         *Str,
+    IN  UINTN         StrSize,
+    IN  CONST CHAR8   *fmt,
+    ms_va_list           args
+);
+
+//
+// For compatibility with previous gnu-efi versions
+//
+#define SPrint      UnicodeSPrint
+#define VSPrint     UnicodeVSPrint
+#define APrint      AsciiPrint
+
 VOID
 ValueToHex (
     IN CHAR16   *Buffer,
@@ -460,6 +734,13 @@ ValueToString (
     IN INT64    v
     );
 
+VOID
+FloatToString (
+    IN CHAR16   *Buffer,
+    IN BOOLEAN  Comma,
+    IN double    v
+    );
+
 VOID
 TimeToString (
     OUT CHAR16      *Buffer,
@@ -521,6 +802,21 @@ LibDeleteVariable (
     );
 
 EFI_STATUS
+LibSetNVVariable (
+    IN CHAR16   *VarName,
+    IN EFI_GUID *VarGuid,
+    IN UINTN	 DataSize,
+    IN VOID     *Data
+    );
+
+EFI_STATUS
+LibSetVariable (
+    IN CHAR16   *VarName,
+    IN EFI_GUID *VarGuid,
+    IN UINTN	 DataSize,
+    IN VOID     *Data
+    );
+EFI_STATUS
 LibInsertToTailOfBootOrder (
     IN  UINT16  BootOption,
     IN  BOOLEAN OnlyInsertIfEmpty
@@ -742,7 +1038,7 @@ OpenSimpleReadFile (
     IN VOID                     *SourceBuffer   OPTIONAL,
     IN UINTN                    SourceSize,
     IN OUT EFI_DEVICE_PATH      **FilePath,
-    OUT EFI_HANDLE              *DeviceHandle,    
+    OUT EFI_HANDLE              *DeviceHandle,
     OUT SIMPLE_READ_FILE        *SimpleReadHandle
     );
 
@@ -783,7 +1079,7 @@ LibGetSystemConfigurationTable(
 
 BOOLEAN
 LibIsValidTextGraphics (
-    IN  CHAR16  Graphic,   
+    IN  CHAR16  Graphic,
     OUT CHAR8   *PcAnsi,    OPTIONAL
     OUT CHAR8   *Ascii      OPTIONAL
     );
@@ -824,39 +1120,44 @@ InitializeGlobalIoDevice (
         IN  EFI_DEVICE_PATH             *DevicePath,
         IN  EFI_GUID                    *Protocol,
         IN  CHAR8                       *ErrorStr,
-        OUT EFI_DEVICE_IO_INTERFACE     **GlobalIoFncs 
+        OUT EFI_DEVICE_IO_INTERFACE     **GlobalIoFncs
         );
 
-UINT32 
+UINT32
 ReadPort (
-        IN  EFI_DEVICE_IO_INTERFACE     *GlobalIoFncs, 
+        IN  EFI_DEVICE_IO_INTERFACE     *GlobalIoFncs,
         IN  EFI_IO_WIDTH                Width,
         IN  UINTN                       Port
         );
 
-UINT32 
+UINT32
 WritePort (
-        IN  EFI_DEVICE_IO_INTERFACE     *GlobalIoFncs, 
+        IN  EFI_DEVICE_IO_INTERFACE     *GlobalIoFncs,
         IN  EFI_IO_WIDTH                Width,
         IN  UINTN                       Port,
         IN  UINTN                       Data
         );
 
-UINT32 
+UINT32
 ReadPciConfig (
-        IN  EFI_DEVICE_IO_INTERFACE     *GlobalIoFncs, 
+        IN  EFI_DEVICE_IO_INTERFACE     *GlobalIoFncs,
         IN  EFI_IO_WIDTH                Width,
         IN  UINTN                       Port
         );
 
-UINT32 
+UINT32
 WritePciConfig (
-        IN  EFI_DEVICE_IO_INTERFACE     *GlobalIoFncs, 
+        IN  EFI_DEVICE_IO_INTERFACE     *GlobalIoFncs,
         IN  EFI_IO_WIDTH                Width,
         IN  UINTN                       Port,
         IN  UINTN                       Data
         );
 
+VOID
+Pause (
+    VOID
+);
+
 extern EFI_DEVICE_IO_INTERFACE  *GlobalIoFncs;
 
 #define outp(_Port, _DataByte)  (UINT8)WritePort(GlobalIoFncs,  IO_UINT8,  (UINTN)_Port, (UINTN)_DataByte)
@@ -873,7 +1174,10 @@ extern EFI_DEVICE_IO_INTERFACE  *GlobalIoFncs;
 #define writepci32(_Addr, _DataByte) (UINT32)WritePciConfig(GlobalIoFncs, IO_UINT32, (UINTN)_Addr, (UINTN)_DataByte)
 #define readpci32(_Addr)             (UINT32)ReadPciConfig(GlobalIoFncs,  IO_UINT32, (UINTN)_Addr)
 
-#define Pause()             WaitForSingleEvent (ST->ConIn->WaitForKey, 0)
 #define Port80(_PostCode)   GlobalIoFncs->Io.Write (GlobalIoFncs, IO_UINT16, (UINT64)0x80, 1, &(_PostCode))
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
